# -*- coding: utf-8 -*- #
"""
Time                2023/01/11 10:13
Author:             mingfeng (SunnyQjm)
Email               mfeng@linux.alibaba.com
File                executor.py
Description:
"""
import asyncio
import base64
import json
import time
import uuid
from asyncer import syncify
from alibabacloud_tea_util import models as util_models
from aliyunsdkcore.request import CommonRequest
from aliyunsdkcore.acs_exception.exceptions import ServerException
from aliyunsdkcore.client import AcsClient
from alibabacloud_ecs20140526.client import Client as Ecs20140526Client
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_ecs20140526 import models as ecs_20140526_models
from .base import BaseChannel, ChannelResult, ChannelException, ChannelCode

ACCESS_KEY = 'LTAI5tCRAPTG9JF68LJ3cvPU'  # 设置 AccessKey ID
ACCESS_KEY_SECRET = 'kQWC6bnGGGDB6uOGKBegjUv5qzcTYN'  # 设置 AccessKey Secret
DEFAULT_EXEC_TIMEOUT = 600000  # 默认执行超时时间 60s


def generate_sts_token(uid: str, access_key: str, access_secret: str,
                       region_id: str = "cn-hangzhou"):
    client = AcsClient(ak=access_key, secret=access_secret,
                       region_id=region_id)
    request = CommonRequest()

    # Fill request params
    request.set_product("Sts")
    request.set_version("2015-04-01")
    request.set_action_name("AssumeRoleWithServiceIdentity")
    request.set_method("POST")
    request.set_protocol_type("https")
    request.add_query_param(
        "RoleArn", f"acs:ram::{uid}:role/aliyunserviceroleforcloudmonitor")
    request.add_query_param("RoleSessionName", "cloudmonitor")
    request.add_query_param("DurationSeconds", "21600")
    request.add_query_param("AssumeRoleFor", uid)
    request.add_query_param("RegionId", region_id)

    try:
        response = client.do_action_with_exception(request)
        response = json.loads(str(response, encoding='utf-8'))
        credentials = response['Credentials']
        return {
            "security_token": credentials.get('SecurityToken'),
            "access_key": credentials.get('AccessKeyId'),
            "access_key_secret": credentials.get('AccessKeySecret'),
            "expiration": credentials.get('Expiration')
        }
    except ServerException as exc:
        raise ChannelException(
            'Instance access failed, please try again later'
        ) from exc


class Channel(BaseChannel):
    """
    CMS channel
    """

    def __init__(self, **kwargs) -> None:
        # Extract required params
        missing_param_list = []
        for require_param in ["uid", "instance", "command", "region"]:
            if require_param not in kwargs:
                missing_param_list.append(require_param)
        if len(missing_param_list) > 0:
            raise ChannelException(
                f"CloudAssistant: Missing parameters: "
                f"{', '.join(missing_param_list)}",
                code=ChannelCode.REQUEST_PARAM_ERROR.value
            )
        self._uid = kwargs.get("uid", None)
        self._instance = kwargs.get("instance", None)
        self._command = kwargs.get("command", None)
        self._region = kwargs.get("region", None)

        # Generate sts token
        self.generate_sts()

    def generate_sts(self):
        try:
            res = generate_sts_token(
                self._uid, ACCESS_KEY, ACCESS_KEY_SECRET, self._region)
            self._access_key = res["access_key"]
            self._secret_key = res["access_key_secret"]
            self._security_token = res["security_token"]
        except Exception as exc:
            raise ChannelException(
                (f"CloudAssistant: Connect to target instance "
                 f"({self._instance}) failed: {str(exc)}"),
                code=ChannelCode.CHANNEL_CONNECT_FAILED.value,
                summary=(
                    f"CloudAssistant: Connect to target instance "
                    f"({self._instance}) failed, please check if you "
                    f"have permission to access the instance")
            ) from exc

    @staticmethod
    def create_client(
            access_key_id: str,
            access_key_secret: str,
            security_token: str,
            region_id: str = "cn-hangzhou"
    ) -> Ecs20140526Client:
        """ Use STS secrets to generate Client

        Args:
            access_key_id:      STS temporary access key
            access_key_secret:  STS temporary access key secret
            security_token:     STS security token
            region_id:          Machine region

        Returns:
            Ecs20140526Client
        """
        config = open_api_models.Config(
            type='sts',
            access_key_id=access_key_id,
            access_key_secret=access_key_secret,
            security_token=security_token,
            region_id=region_id,
        )
        return Ecs20140526Client(config)

    @staticmethod
    def initial(**kwargs) -> ChannelResult:
        return ChannelResult()

    @staticmethod
    async def initial_async(**kwargs) -> ChannelResult:
        return ChannelResult()

    def run_command(self, **kwargs) -> ChannelResult:
        """Execute command on remote machine

        Args:
            **kwargs:

        Returns:

        """
        return syncify(self.run_command_async, raise_sync_error=False)(
            **kwargs
        )

    async def _do_get_result(self, client: Ecs20140526Client, invoke_id: str,
                             region: str, timeout: int):
        """"""
        channel_result = ChannelResult(
            code=ChannelCode.SERVER_ERROR.value,
            err_msg="CloudAssistant: Get command result failed",
            result="CloudAssistant: Get command result failed"
        )
        expire_time = timeout + time.time()
        describe_invocation_results_request = \
            ecs_20140526_models.DescribeInvocationResultsRequest()
        describe_invocation_results_request.invoke_id = invoke_id
        describe_invocation_results_request.region_id = region
        runtime = util_models.RuntimeOptions()
        try:
            while True:
                res = await client. \
                    describe_invocation_results_with_options_async(
                        describe_invocation_results_request, runtime
                    )
                # https://next.api.aliyun.com/api/Ecs/2014-05-26/DescribeInvocationResults?lang=PYTHON&sdkStyle=dara&tab=DOC
                # InvocationStatus =>
                # ["Pending", "Invalid", "Aborted", "Running", "Success",
                #  "Failed", "Error", "Timeout", "Cancelled", "Stopping",
                #  "Terminated", "Scheduled"]
                if len(res.body.invocation.invocation_results.invocation_result) <= 0:
                    raise ChannelException(
                        (f"CloudAssistant: Unexpected error, invocation "
                         f"results empty!"),
                        code=ChannelCode.CHANNEL_EXEC_FAILED.value
                    )
                invoke_result = \
                    res.body.invocation.invocation_results. \
                    invocation_result[0]
                # print(invoke_result)
                if invoke_result.invocation_status == "Pending" or \
                        invoke_result.invocation_status == "Running":
                    await asyncio.sleep(1)
                    if time.time() > expire_time:
                        raise ChannelException(
                            (f"CloudAssistant: Execute command timeout ("
                             f"invoke_id = {invoke_id}, instance = "
                             f"{self._instance})"),
                            code=ChannelCode.CHANNEL_CONNECT_TIMEOUT.value,
                        )
                elif invoke_result.invocation_status == "Success":
                    output = base64.b64decode(
                        invoke_result.output).decode("utf-8")
                    channel_result.result = output
                    channel_result.code = ChannelCode.SUCCESS.value
                    channel_result.err_msg = ""
                    break
                elif invoke_result.invocation_status == "Failed":
                    output = base64.b64decode(
                        invoke_result.output).decode("utf-8")
                    channel_result.result = (
                        "CloudAssistant: Execute command failed (invoke_id "
                        f"= {invoke_id}, instance = {self._instance})")
                    channel_result.code = ChannelCode.CHANNEL_EXEC_FAILED.value
                    channel_result.err_msg = (
                        "CloudAssistant: Execute command failed (invoke_id "
                        f"= {invoke_id}, instance = {self._instance}) => "
                        f"{output}")
                    break
                elif invoke_result.invocation_status == "Timeout":
                    raise ChannelException(
                        (f"CloudAssistant: Connect timeout (invoke_id = "
                         f"{invoke_id}, instance = {self._instance})"),
                        code=ChannelCode.CHANNEL_CONNECT_TIMEOUT.value,
                    )
                else:
                    raise ChannelException(
                        (f"CloudAssistant: Unexpected invocation status "
                         f"(invocation_status "
                         f"= {invoke_result.invocation_status}, invoke_id "
                         f"= {invoke_id}, instance = {self._instance})"),
                        code=ChannelCode.CHANNEL_EXEC_FAILED.value
                    )
        except ChannelException as ce:
            raise ce
        except Exception as exc:
            raise ChannelException(
                (f"CloudAssistant: Get invocation result failed (invoke_id "
                 f"= {invoke_id}, instance = {self._instance}) => {str(exc)}"),
                code=ChannelCode.CHANNEL_EXEC_FAILED.value,
                summary=(
                    f"CloudAssistant: Get invocation result failed (invoke_id "
                    f"= {invoke_id}, instance = {self._instance})"),
            ) from exc
        return channel_result

    async def _do_run_command(self, command: str, **kwargs) -> ChannelResult:
        client = Channel.create_client(
            self._access_key, self._secret_key, self._security_token, self._region
        )
        run_command_request = ecs_20140526_models.RunCommandRequest()
        run_command_request.region_id = self._region
        run_command_request.type = "RunShellScript"
        run_command_request.command_content = command
        run_command_request.instance_id = [self._instance]
        timeout = int(kwargs.get(
            "timeout", DEFAULT_EXEC_TIMEOUT) / 1000)
        run_command_request.timeout = timeout
        runtime = util_models.RuntimeOptions()
        invoke_id = "Unknown"
        try:
            res = await client.run_command_with_options_async(
                run_command_request, runtime)
            invoke_id = res.body.invoke_id
            channel_result = await self._do_get_result(
                client, invoke_id, self._region, timeout
            )
        except ChannelException as ce:
            raise ce
        except Exception as exc:
            raise ChannelException(
                (f"CloudAssistant: Connect failed (invoke_id = {invoke_id}, "
                 f"instance = {self._instance}) => {str(exc)}"),
                code=ChannelCode.CHANNEL_CONNECT_FAILED.value,
                summary=(f"CloudAssistant: Connect failed (invoke_id = "
                         f"{invoke_id}, instance = {self._instance})")
            ) from exc
        return channel_result

    async def run_command_async(self, **kwargs) -> ChannelResult:
        """Execute command on remote machine by async

        Args:
            **kwargs:

        Returns:

        """
        timeout_ms = kwargs.get("timeout", DEFAULT_EXEC_TIMEOUT)
        start_time = time.time() * 1000
        # 1. 首先执行命令，并把终端输出重定向到一个临时文件
        tmp_file_name = uuid.uuid4().hex
        channel_result = await self._do_run_command(
            f"({self._command}) > /tmp/{tmp_file_name} 2>&1; "
            f'if [ "$?" -ne "0" ]; then cat /tmp/{tmp_file_name}; exit 1;'
            f'elif [ ! -s /tmp/{tmp_file_name} ]; then exit 0; else '
            f"split -b 20KB /tmp/{tmp_file_name} /tmp/{tmp_file_name}_chunk; "
            f"rm /tmp/{tmp_file_name}; ls /tmp/{tmp_file_name}_chunk*;"
            f"fi",
            **kwargs
        )
        if channel_result.code != 0:
            return channel_result

        channel_result.result = channel_result.result.strip()
        if not channel_result.result:
            return channel_result

        # 2. 分片取回结果（每个分片 20KB），同时删除对应的分片
        timeout_ms = timeout_ms - ((time.time() * 1000) - start_time)
        if timeout_ms <= 0:
            raise ChannelException(
                (f"CloudAssistant: Execute command timeout ("
                 f"instance = "
                 f"{self._instance})"),
                code=ChannelCode.CHANNEL_CONNECT_TIMEOUT.value,
            )
        kwargs["timeout"] = timeout_ms
        chunks = channel_result.result.strip().split("\n")
        tasks = []
        for chunk in chunks:
            tasks.append(
                self._do_run_command(f"cat {chunk} && rm {chunk}", **kwargs))

        channel_result = ChannelResult(
            code=ChannelCode.SUCCESS.value,
            err_msg="",
            result=""
        )
        total_task = len(tasks)
        for i in range(0, len(tasks), 1):
            for res in await asyncio.gather(*tasks[i: min(i + 1, total_task)]):
                if res.code != 0:
                    return res
                else:
                    channel_result.result += res.result

        return channel_result
