# -*- coding: utf-8 -*- #
"""
Time                2023/01/11 10:13
Author:             mingfeng (SunnyQjm)
Email               mfeng@linux.alibaba.com
File                executor.py
Description:
"""

import logging
import json
import time
import asyncio
from asyncer import syncify
from alibabacloud_eci_inner20190910.client import Client
from alibabacloud_eci_inner20190910.models import (
    OpsListEcisRequest, OpsRunCommandRequest, OpsDescribeCommandResultRequest
)
from alibabacloud_tea_openapi.models import Config
from .base import BaseChannel, ChannelException, ChannelResult, ChannelCode

logger = logging.getLogger(__name__)

ACCESS_KEY = 'LTAI5tC85b1AwncETiGpLt3G'
ACCESS_KEY_SECRET = 'WuwrCykCnkW8UAwXf6nqZWycFpONNS'
DEFAULT_EXEC_TIMEOUT = 600000                          # 默认执行超时时间 60s
SLEEP_TIME = 1

PRE_CMD = "if [ \"\" == \"\" ]; then wget -q -O /tmp/sysak.rpm https://osdh-%s.oss-%s-internal.aliyuncs.com/eci/sysak-1.4.0-1.al7.x86_64.rpm && rpm -i --nodeps --force /tmp/sysak.rpm && rm -f /tmp/sysak.rpm; fi;"


class Channel(BaseChannel):
    """
    ECI 通道
    """

    def __init__(self, **kwargs):
        # Extract required params
        missing_param_list = []
        for require_param in ["instance", "command"]:
            if require_param not in kwargs:
                missing_param_list.append(require_param)
        if len(missing_param_list) > 0:
            raise ChannelException(
                f"CloudAssistant: Missing parameters: {', '.join(missing_param_list)}",
                code=ChannelCode.REQUEST_PARAM_ERROR.value
            )

        self._region_id = kwargs.get("region", "cn-hangzhou")
        self._instance = kwargs.get("instance", "")
        self._command = kwargs.get("command", "")

    @staticmethod
    def initial(**kwargs) -> ChannelResult:
        return ChannelResult()

    @staticmethod
    def create_client(access_key: str, access_key_secret: str,
                      region_id: str = "cn-hangzhou") -> Client:
        config = Config(access_key_id=access_key,
                        access_key_secret=access_key_secret,
                        region_id=region_id,
                        endpoint='eci-share.{}.aliyuncs.com'.format(region_id))
        return Client(config=config)

    def run_command(self, **kwargs) -> ChannelResult:
        return syncify(self.run_command_async, raise_sync_error=False)(
            **kwargs
        )

    async def _get_region_by_eci_id_async(self, client: Client, eci_id: str) -> str:
        request = OpsListEcisRequest()
        request.container_group_ids = json.dumps([eci_id])
        request.region_id = self._region_id
        try:
            res = await client.ops_list_ecis_async(request)
            if not res or not res.body or not res.body.container_groups or                     len(res.body.container_groups) < 1:
                raise Exception(f"ops_list_ecis_async response empty: {res}")
            return res.body.container_groups[0].region_id
        except Exception as exc:
            raise ChannelException(
                (f"ECI Channel: Get region by eci id failed"
                 f"(instance = {eci_id}, region_id = {self._region_id}) => {str(exc)}"),
                code=ChannelCode.CHANNEL_CONNECT_FAILED.value,
                summary=(f"ECI Channel: Get region by eci id failed"
                         f"(instance = {eci_id}, region_id = {self._region_id})")
            ) from exc

    async def run_command_async(self, **kwargs) -> ChannelResult:
        channel_result = ChannelResult(
            code=ChannelCode.SERVER_ERROR.value,
            err_msg="ECI Channel: Run command error",
            result="ECI Channel: Run command error"
        )
        client = Channel.create_client(ACCESS_KEY, ACCESS_KEY_SECRET)
        run_command_request = OpsRunCommandRequest()
        run_command_request.container_group_id = self._instance
        run_command_request.region_id = await self._get_region_by_eci_id_async(client, self._instance)
        pre_cmd = PRE_CMD % (run_command_request.region_id,
                             run_command_request.region_id)
        run_command_request.command = pre_cmd + self._command
        timeout = int(kwargs.get(
            "timeout", DEFAULT_EXEC_TIMEOUT) / 1000)
        expire_time = timeout + time.time()
        try:
            run_command_response = await client.ops_run_command_async(run_command_request)
            if not run_command_response or not run_command_response.body:
                raise Exception((f"ops_run_command_async response empty => "
                                "Please check if the agent is deployed on "
                                 "the instance, or upgrade the agent and try again."))
            describe_command_result_request = OpsDescribeCommandResultRequest()
            describe_command_result_request.command_id = run_command_response.body.command_id
            describe_command_result_request.container_group_id = self._instance
            describe_command_result_request.region_id = run_command_request.region_id
            try:
                while True:
                    res = await client.ops_describe_command_result_async(
                        describe_command_result_request
                    )
                    if res.body.status == "Running":
                        await asyncio.sleep(SLEEP_TIME)
                        if time.time() > expire_time:
                            raise ChannelException(
                                ("ECI Channel: Execute command timeout"
                                 f"(instance = {self._instance}, region_id "
                                 f"= {run_command_request.region_id})"),
                                code=ChannelCode.CHANNEL_CONNECT_TIMEOUT.value,
                            )
                    elif res.body.status == "Succeed":
                        channel_result.result = res.body.result
                        channel_result.code = ChannelCode.SUCCESS.value
                        channel_result.err_msg = ""
                        break
                    else:
                        channel_result.result = ("ECI Channel: Execute command failed"
                                                 f"(instance = {self._instance}, region_id "
                                                 f"= {run_command_request.region_id})")
                        channel_result.code = ChannelCode.CHANNEL_EXEC_FAILED.value
                        channel_result.err_msg = ("ECI Channel: Execute command failed"
                                                  f"(instance = {self._instance}, region_id "
                                                  f"= {run_command_request.region_id}) => {res.body.result}")
                        break
            except ChannelException as ce:
                raise ce
            except Exception as exc:
                raise ChannelException(
                    (f"ECI Channel: Get command execute result failed "
                     f"(instance = {self._instance}, region_id "
                     f"= {run_command_request.region_id}) => {str(exc)}"),
                    code=ChannelCode.CHANNEL_EXEC_FAILED.value,
                    summary=(f"ECI Channel: Get command execute result failed "
                             f"(instance = {self._instance}, region_id "
                             f"= {run_command_request.region_id})")
                ) from exc
        except ChannelException as ce:
            raise ce
        except Exception as exc:
            raise ChannelException(
                (f"ECI Channel: Connect failed "
                 f"(instance = {self._instance}, region_id "
                 f"= {run_command_request.region_id}) => {str(exc)}"),
                code=ChannelCode.CHANNEL_CONNECT_FAILED.value,
                summary=(f"ECI Channel: Connect failed "
                         f"(instance = {self._instance}, region_id "
                         f"= {run_command_request.region_id})"),
            ) from exc
        return channel_result

