from cec_base.event import Event
from cec_base.consumer import Consumer
from cec_base.producer import Producer
from cec_base.cec_client import MultiConsumer, CecAsyncConsumeTask
from clogger import logger
from conf.settings import *
from app.database import SessionLocal
from app.crud import create_task_id_and_uuid
from app.schemas import UuidToTaskCreate
from sysom_utils import CecTarget, SysomFramework

class DiagnosisListener(MultiConsumer):

    def __init__(self) -> None:
        super().__init__(
            YAML_CONFIG.get_cec_url(CecTarget.PRODUCER),
            custom_callback=self.on_receive_event,
        )
        self.append_group_consume_task(
            SYSOM_CEC_DIAGNOSIS_TASK_CREATED,
            "sysom_diagnosis",
            Consumer.generate_consumer_id(),
            ensure_topic_exist=True,
        )
        self._producer: Producer = SysomFramework.cec_producer()

    def _delivery(self, topic: str, value: dict):
        self._producer.produce(topic, value)
        self._producer.flush()

    def _create_task(self, uuid_to_task: dict):
        with SessionLocal() as db:
            create_task_id_and_uuid(db, UuidToTaskCreate(**uuid_to_task))

    def on_receive_event(self, event: Event, task: CecAsyncConsumeTask):
        event_value = event.value
        logger.warning(type(event_value), event_value)
        try:
            assert isinstance(event_value, dict)
            if task.topic_name == SYSOM_CEC_DIAGNOSIS_TASK_CREATED:
                _uuid = event_value["params"]["uuid"]
                _task_id = event_value["task_id"]
                param = {'uuid' : _uuid, 'task_id' : _task_id}
                self._create_task(param)
            else:
                logger.warning(
                    f"Received not expect topic data, topic = {task.topic_name}"
                )
        except Exception as e:
            logger.exception(e)
        finally:
            task.ack(event)

