from typing import List
from clogger import logger
from fastapi import APIRouter, Request, Depends
from fastapi.responses import RedirectResponse
from sqlalchemy.orm import Session
from app.crud import get_task_id_by_uuid
from app.database import get_db
from app.schemas import GetTask
from app.query import UuidToTaskQueryParams



router = APIRouter()


@router.get("/task_url", response_model=str)
async def get_task_url(
    uuid: str, db: Session = Depends(get_db)
):
    uuid_to_task = get_task_id_by_uuid(
        uuid, db
    )
    task_id = uuid_to_task.task_id
    return RedirectResponse(f"/diagnose/detail/{task_id}")

@router.get("/task_id", response_model=str)
async def get_task_id(
    uuid: str, db: Session = Depends(get_db)
):
    uuid_to_task = get_task_id_by_uuid(
        uuid, db
    )
    return uuid_to_task.task_id